@@ -1,5 +1,5 @@
 # -*- coding:utf8 -*-
-######################## BEGIN LICENSE BLOCK ########################
+# ####################### BEGIN LICENSE BLOCK ########################
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
@@ -14,7 +14,7 @@
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 # 02110-1301 USA
-######################### END LICENSE BLOCK #########################
+# ######################## END LICENSE BLOCK #########################
 """
 This module handels the entire upload and some argument and response checking
 """
@@ -27,9 +27,10 @@ from requests_toolbelt import MultipartEncoder, MultipartEncoderMonitor
 
 
 from picuplib.checks import (check_resize, check_rotation, check_noexif,
-                             check_response, check_if_redirect)
+                             check_response, check_if_redirect, check_callback)
 from picuplib.globals import API_URL, USER_AGENT
 
+
 class Upload(object):
     """
     Class based wrapper for uploading.
@@ -43,18 +44,25 @@ class Upload(object):
         Allowed values are 00, 90, 180, 270.(optional)
     :param boolean noexif: set to True when exif data should be purged.\
         (optional)
+    :param function callback: function witch will be called after every read. \
+        Need to take one argument. you can use the len function to \
+        determine the body length and call bytes_read(). Only for local Upload!
 
     :ivar str resize:
     :ivar str rotation:
     :ivar boolean noexif: If true exif data will be deleted
+    :ivar function callback:
     """
 
-    def __init__(self, apikey, resize='og', rotation='00', noexif=False):
+    def __init__(self, apikey, resize='og', rotation='00', noexif=False,
+                 callback=None):
         self._apikey = apikey
         self._resize = resize
         self._rotation = rotation
         self._noexif = noexif
 
+        self._callback = callback
+
     @property
     def resize(self):
         """getter for _resize"""
@@ -88,7 +96,19 @@ class Upload(object):
         check_noexif(value)
         self._noexif = value
 
-    def upload(self, picture, resize=None, rotation=None, noexif=None):
+    @property
+    def callback(self):
+        """ getter for _callback"""
+        return self._callback
+
+    @callback.setter
+    def callback(self, value):
+        """setter for _callback"""
+        check_callback(value)
+        self._callback = value
+
+    def upload(self, picture, resize=None, rotation=None, noexif=None,
+               callback=None):
         """
         wraps upload function
 
@@ -99,6 +119,9 @@ class Upload(object):
             Allowed values are 00, 90, 180, 270.(optional)
         :param boolean noexif: set to True when exif data should be purged.\
             (optional)
+        :param function callback: function witch will be called after every read. \
+            Need to take one argument. you can use the len function to \
+            determine the body length and call bytes_read().
 
         """
         if not resize:
@@ -107,8 +130,10 @@ class Upload(object):
             rotation = self._rotation
         if not noexif:
             noexif = self._noexif
+        if not callback:
+            callback = self._callback
 
-        return upload(self._apikey, picture, resize, rotation, noexif)
+        return upload(self._apikey, picture, resize, rotation, noexif, callback)
 
     def remote_upload(self, picture_url, resize=None,
                       rotation=None, noexif=None):
@@ -135,6 +160,7 @@ class Upload(object):
         return remote_upload(self._apikey, picture_url,
                              resize, rotation, noexif)
 
+
 def punify_filename(filename):
     """
     small hackisch workaround for unicode problems with the picflash api
@@ -155,10 +181,13 @@ def upload(apikey, picture, resize='og', rotation='00', noexif=False,
         Allowed values are 00, 90, 180, 270.(optional)
     :param boolean noexif: set to True when exif data should be purged.\
         (optional)
-
+    :param function callback: function witch will be called after every read. \
+        Need to take one argument. you can use the len function to determine \
+        the body length and call bytes_read().
     """
     check_rotation(rotation)
     check_resize(resize)
+    check_callback(callback)
 
     post_data = compose_post(apikey, resize, rotation, noexif)
 
@@ -167,6 +196,7 @@ def upload(apikey, picture, resize='og', rotation='00', noexif=False,
 
         return do_upload(post_data, callback)
 
+
 def remote_upload(apikey, picture_url, resize='og',
                   rotation='00', noexif=False):
     """
@@ -193,6 +223,7 @@ def remote_upload(apikey, picture_url, resize='og',
 
     return do_upload(post_data)
 
+
 def resolve_redirect(url):
     """
     recursively resolves redirects
@@ -221,6 +252,7 @@ def compose_post(apikey, resize, rotation, noexif):
 
     return post_data
 
+
 def do_upload(post_data, callback=None):
     """
     does the actual upload also sets and generates the user agent string
@@ -229,11 +261,8 @@ def do_upload(post_data, callback=None):
     encoder = MultipartEncoder(post_data)
     monitor = MultipartEncoderMonitor(encoder, callback)
 
-
-
     headers = {'User-Agent': USER_AGENT, 'Content-Type': monitor.content_type}
     response = post(API_URL, data=monitor, headers=headers)
     check_response(response)
 
     return response.json()[0]
-
